﻿using UnityEngine;
using System.Collections;

public class EnemySpawner : MonoBehaviour {
	
	//This allows us to drag and drop the actual enemy prefab from our Assets onto the EnemyFormation game object in the inspector.
	public GameObject enemyPrefab;
	
	//Variable we use to determine the direction our enemy formation is moving.
	public bool movingRight = true;
	
	//We specify our bounding box for our EnemyFormation so we can make sure it stays within the gamespace.
	public float width = 10f;
	public float height = 5f;
	
	//We specify the speed of the movement of our EnemyFormation.
	public float speed = 5f;
	
	public float spawnDelay = 1f;
	
	//These represent the left and right bounding walls of our game space.	
	private Vector3 leftmost;
	private Vector3 rightmost;
	
	//These represent the updated left and right bounding walls of our gamespace that includes padding for our ship sprites on the left most and right most edges of our EnemyFormation.
	//It make sense for future scripts to make these public to take advantage of.
	private float xmin;
	private float xmax;
	
	// Use this for initialization
	void Start () {
		//Here we spawn the initial enemies when the game loads.  Refer to the comments in this method below for more detail.
		SpawnUntilFull ();
		
		//We want to calculate the size of the gamespace X axis and restrict the EnemyFormation so it does not go outside those values.
		//Here we set the leftmost and rightmost values of the viewport where the bottom left of the viewport is (0,0) and the top right is (1,1).
		//The 3rd value is the Z value and since this is 2D we set it to 0 so it isn't changed from the value set manually in the inspector for the prefab.
		leftmost = Camera.main.ViewportToWorldPoint (new Vector3 (0, 0, 0));
		rightmost = Camera.main.ViewportToWorldPoint (new Vector3 (1, 0, 0));
		
		//Update the viewport boundaries to include padding for our left most and right most sprites in our EnemyFormation.
		//The (width / 2) represents the padding we need from the center of the EnemyFormation.
		xmin = leftmost.x + (width / 2);
		xmax = rightmost.x - (width / 2);
	}
	
	// Update is called once per frame
	void Update () {
		//Incrementally move the formation in whatever direction left or right.
		//Check the value of movingRight to start moving the formation.
		//If true, then move it right.  If not, then move it left.
		if (movingRight) {
			transform.position += Vector3.right * speed * Time.deltaTime;	
		} else {
			transform.position += Vector3.left * speed * Time.deltaTime;
		}
		
		//Clamp the left and right viewport values so the EnemyFormation cannot pass outside the gamespace walls.
		float xnew = Mathf.Clamp(transform.position.x, xmin, xmax);
		
		//Finalize the calculations and move the EnemyFormation.
		transform.position = new Vector3(xnew, transform.position.y, transform.position.z);
		
		//Determine if the EnemyFormation hits either the left gamespace wall or right gamespace wall, then change direction accordingly.
		if (transform.position.x <= xmin) {
			movingRight = true;
		} else if (transform.position.x >= xmax) {
			movingRight = false;
		}
		
		/* Used for debugging
		TODO: Remove this before production release.
		print ("Transform " + transform.position.x);
		print ("xmax " + xmax);
		print ("xmin " + xmin);
		*/
		
		//Determine if all Enemies in the game space are dead.  If so, spawn more Enemies.
		if (AllEnemiesDead ()) {
			SpawnUntilFull ();
		}
	}
	
	public void SpawnEnemies() {
		//If we ran this code with just the Instantiate statement, the enemy sprite(s) will load into the game, but to keep our hierarchy tidy we want to spawn enemies within our Position prefab.
		//We store the Instantiated enemy sprite into another game object called "enemy" and then move it into the Position prefab in the hierarchy.
		//Each Position prefab is placed in a unique area on the game space.  Running the foreach statement will check every Position prefab and spawn an enemy at its location.
		//We aren't spawning it into the parent (EnemyFormation) we are spawning it into all of its unique children which are the Position game objects.
		foreach (Transform child in transform) {
			GameObject enemy = Instantiate(enemyPrefab, child.transform.position, Quaternion.identity) as GameObject;
			enemy.transform.parent = child;
			//The code below for Instantiate only works with Unity 5.4 and later. You will have to use the longer way above until then.
			//Instantiate(enemyPrefab, new child.transform.position, Quaternion.identity, transform);
		}
	}
	
	//Determine if there is a Position child that is empty.  If so, return that Position.
	//We use this to help identify when a Position child becomes available for us to spawn an Enemy.
	Transform NextFreePosition() {
		//Iterate through each of the Position children and return any that are empty.
		foreach (Transform formationChildren in transform) {
			if (formationChildren.childCount == 0) {
				return formationChildren;
			} 
			
		}
		//If all Positions are full, then return null.
		return null;
	}
	
	public void SpawnUntilFull() {
		Transform freePosition = NextFreePosition();
		if (freePosition) {
			GameObject enemy = Instantiate(enemyPrefab, freePosition.position, Quaternion.identity) as GameObject;
			enemy.transform.parent = freePosition;
			Invoke("SpawnUntilFull", spawnDelay);
		}
	}
	/*	
	public void SpawnUntilFull() {
		Transform freePosition = NextFreePosition ();
		if (freePosition) {
			GameObject enemy = Instantiate(enemyPrefab, freePosition.position, Quaternion.identity) as GameObject;
			enemy.transform.parent = freePosition;
			//The code below for Instantiate only works with Unity 5.4 and later. You will have to use the longer way above until then.
			//Instantiate(enemyPrefab, new child.transform.position, Quaternion.identity, transform);
		}
		
		if (NextFreePosition ()) {
			Invoke ("SpawnUntilFull", spawnDelay);
		}
	}
	*/
	
	//Determine if all the Position children are empty.
	public bool AllEnemiesDead(){
		//Iterate through each Position child and determine if any children exist.  If they do, then return false.  If they don't, then return true.
		foreach (Transform formationChildren in transform) {
			if (formationChildren.childCount > 0) {
				return false;
			} 
			
		}
		
		return true;
	}
	
	//We use this to draw our bounding box for our EnemyFormation so we can see it in the editor.
	//We calculate the radius by using a Vector3 of the width and height variables which specify our EnemyFormation size.
	public void OnDrawGizmos() {
		Gizmos.DrawWireCube(transform.position, new Vector3(width, height, 0));
	}
	
	
}
