﻿using UnityEngine;
using System.Collections;

public class PlayerController : MonoBehaviour {

	//We create a public variable to control the speed of our player's ship that can be accessed in the inspector for play testing.
	public float speed = 15f;
	
	//This is used to expose on our script the ability to specify what laser sprite prefab we want to use for our Player ship.
	public GameObject projectile;
	
	//This controls the speed at which we want our "projectile" to shoot from our Player ship.
	public float projectileSpeed = 10f;
	
	//This gives us more control on the rate of fire of our projectiles.
	public float fireRate = 0.3f;
	
	//These are variables only relevant to this script to calculate the minimum (left viewport wall) and maximum (right viewport weall) X value for our gamespace.
	private float xmin;
	private float xmax;
	
	//Used to find HALF the width of our ship sprite so we can create padding to ensure the ships don't go outside the game space.
	private float spriteSize;
		
	//These represent the left and right bounding walls of our gamespace.	
	private Vector3 leftmost;
	private Vector3 rightmost;
	
	//This is used to set the scale (size) of our sprite for things like power up or explosions (whatever we want).
	//I made this public so I could play with in the inspector, but you can change it programmatically however you want.
	public float setScale = 1;
	
	//This is used to define the health of our Player's ship.
	//You can make it public if needed to watch it while testing.	
	private float health = 900f;
	
		
	void Start () {
		//We want to calculate the size of the gamespace X axis and restrict the ship sprite so it does not go outside those values.
		//Here we set the leftmost and rightmost values of the viewport where the bottom left of the viewport is (0,0) and the top right is (1,1).
		//The 3rd value is the Z value and since this is 2D we set it to 0 so it isn't changed from the value set manually in the inspector for the prefab.
		leftmost = Camera.main.ViewportToWorldPoint (new Vector3 (0, 0, 0));
		rightmost = Camera.main.ViewportToWorldPoint (new Vector3 (1, 0, 0));
		
		//Grab half the Player1_Ship sprite's width to use as padding for the left and right viewport walls to ensure the ship does not go outside the game space.
		//The sprite.bounds.extents represents HALF of a sprite's given width, height, or depth.  Here we are interested in the width or X value.
		spriteSize = GetComponent<SpriteRenderer> ().sprite.bounds.extents.x;
		
		//Update the viewport boundaries to include the padding for our sprite.
		xmin = leftmost.x + spriteSize;
		xmax = rightmost.x - spriteSize;
	}
	
	void Update () {
		//Here we are capturing the user pressing (or holding down) the left arrow key to move our ship to the left.
		if (Input.GetKey (KeyCode.LeftArrow)) {
			//We use Time.deltatime multiplied by our speed variable multiplied by our Unity default Vector3 structure with the left method to move our ship left.
			//TODO:  This method allows the ship movement to be frame rate independent - find out exactly how that works.
			transform.position += Vector3.left * speed * Time.deltaTime;
			
		//We do the same thing here, except for the right arrow to move our ship to the right we use Vector3.right.
		} else if (Input.GetKey (KeyCode.RightArrow)) {
			transform.position += Vector3.right * speed * Time.deltaTime;
		}
		
		//Clamp the left and right viewport values so the ship cannot pass outside the gamespace walls using our updated xmin and xmax values that include our ship padding.
		float xnew = Mathf.Clamp(transform.position.x, xmin, xmax);
		
		//Finalize the calculations and move the ship.
		transform.position = new Vector3(xnew, transform.position.y, transform.position.z);
		
		//Check the value of our setScale variable in order to determine the ship sprite's actual scale.
		setSpriteScale ();
		
		//This controls the Player ship's laser (projectile variable).
		if (Input.GetKeyDown(KeyCode.Space)) {
			//This controls our firing rate when the Space key is pressed or held down.
			InvokeRepeating("fireProjectile", 0.000001f, fireRate);
		}
			//This stops our firing rate when the Space key is let go
		if (Input.GetKeyUp(KeyCode.Space)) {
			CancelInvoke("fireProjectile");
		}
		
	}
	
	//This method is used to actually fire the projectile from our ship with our projectileSpeed value.
	void fireProjectile() {
	
		//We use this to move the layer above our Player ship to avoid the collision of the laser upon instantiation.
		Vector3 startPosition = transform.position + new Vector3(0, 1f, 0);
		
		//Here we actual spawn an instance of the Player laser prefab and apply our velocity to it using projectileSpeed.		
		GameObject playerLaser = Instantiate(projectile, startPosition, Quaternion.identity) as GameObject;
		playerLaser.rigidbody2D.velocity = new Vector3 (0, projectileSpeed, 0);
		
	}
	
	void OnTriggerEnter2D(Collider2D collider)  {
		//Used for debugging.
		//TODO:  Remove before production release.
		print ("Collision");
		
		//We call upon the Projectiles class to create an instance of a projectile so we can use it to detect collision with our enemy ships.
		Projectiles missile = collider.gameObject.GetComponent<Projectiles>();
		
		//Used for debugging.
		//TODO:  Remove before production release.
		print (missile.name);
		
		//If that projectile exists in the game space, then decrease the health of the enemy ship that is hit by the damage returned by the projectile using the GetDamage() method from the Projectiles class.
		//Run the Hit method of the projectile for any specific projectile properties to be run.
		//If our enemy ship's health is less than or equal to 0, then destroy it.
		if (missile) {
			//Used for debugging.
			//TODO:  Remove before production release.
			print ("Hit by projectile");
			health -= missile.GetDamage();
			
			//This Hit() method of the Projectiles class destroys the laser game object from the game space.
			missile.Hit();
			
			//This will destroy the Enemy ship the laser hits if it determines its health is less than or equal to zero.
			if (health <= 0) {
				Destroy(gameObject);
			}
		}
	}
	
	void setSpriteScale() {
		//We use this method to check the value of our setScale variable and update the scale of our ship sprite to match it.
		//By default the ship starts with a scale of 1 (normal scale).
		if (setScale != 1) {
			//If the setScale variable is not 1, then update the actual scale of the sprite to the value of the setScale variable.
			//We don't use the Z value here since we are using a 2D sprite.
			transform.localScale = new Vector3(setScale, setScale, 0); 
			
			//Update the new boundaries for our viewport to match the updated scale of our sprite.
			xmin = leftmost.x + (spriteSize * setScale);
			xmax = rightmost.x - (spriteSize * setScale);
			
		} else {
			//If the setScale variable is set to 1, then we make sure to bring the ship back to its normal scale we set back in the Start() method.
			//We don't use the Z value here since we are using a 2D sprite.
			transform.localScale = new Vector3(setScale, setScale, 0);
			xmin = leftmost.x + spriteSize;
			xmax = rightmost.x - spriteSize;
		} 
	}
	
}
